//---------------------------------------------------------------------------
// LMConnection: Represents one connection to the lock manager
//---------------------------------------------------------------------------
import java.net.Socket;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.EOFException;

public class LMConnection implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public LMConnection(Socket socket, SyncQueue requestQueue)
    {
	this.socket = socket;
	this.requestQueue = requestQueue;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	InputStream is = null;
	LMRequestPacket packet = null;

	try{
	    while(true){
		is = this.socket.getInputStream();
		packet = new LMRequestPacket(is, this.socket);
		this.requestQueue.enqueue(packet);
		packet = null;
		is = null;
	    }
	}catch(EOFException e){
	    System.out.println("Client done; closing socket");
	    this.closeSocket(socket);
	    socket = null;
	}catch(IOException e){
	    System.out.println("" + e);
	    e.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Close the current socket
    //-----------------------------------------------------------------------
    private void closeSocket(Socket socket)
    {
	InputStream is = null;
	OutputStream os = null;

	if(socket != null){
	    try{
		is = socket.getInputStream();
		is.close();
	    }catch(IOException e){
		// Do nothing
	    }
	    try{
		os = socket.getOutputStream();
		os.close();
	    }catch(IOException e){
		// Do nothing
	    }
	    try{
		socket.close();
	    }catch(IOException e){
		// Do nothing
	    }
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected Socket socket;
    protected SyncQueue requestQueue;
}
