import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.ServerSocket;

//---------------------------------------------------------------------------
// LMLockServer: Lock server starting point
//---------------------------------------------------------------------------
public class LMLockServer{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public static final int LM_PORT_NUM = 5555;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public LMLockServer()
    {
	this.requestQueue = new SyncQueue();
	this.handleThread = new LMHandleThread(this.requestQueue);
    }

    //-----------------------------------------------------------------------
    // Start the main server loop
    //-----------------------------------------------------------------------
    public void enterServerLoop()
    {
	Socket socket = null;
	ServerSocket serverSocket = null;

	try{
	    serverSocket = new ServerSocket(LMLockServer.LM_PORT_NUM);
	}catch(IOException e){
	    System.err.println("" + e);
	    Assert.affirm(false);
	}
	while(true){
	    try{
		socket = serverSocket.accept();
		socket.setTcpNoDelay(true);
		new LMConnection(socket, this.requestQueue);
		socket = null;
	    }catch(IOException e){
		System.err.println("" + e);
		this.closeSocket(socket);
		assert(false);
	    }
	}
    }

    //-----------------------------------------------------------------------
    // Program starting point
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	LMLockServer lockServer = null;

	lockServer = new LMLockServer();
	lockServer.enterServerLoop();
    }

    //-----------------------------------------------------------------------
    // Close the current socket
    //-----------------------------------------------------------------------
    private void closeSocket(Socket socket)
    {
	InputStream is = null;
	OutputStream os = null;

	if(socket != null){
	    try{
		is = socket.getInputStream();
		is.close();
	    }catch(IOException e){
		// Do nothing
	    }
	    try{
		os = socket.getOutputStream();
		os.close();
	    }catch(IOException e){
		// Do nothing
	    }
	    try{
		socket.close();
	    }catch(IOException e){
		// Do nothing
	    }
	}
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    SyncQueue requestQueue;
    LMHandleThread handleThread;
}
