import java.util.Hashtable;

//---------------------------------------------------------------------------
// LMLockStateTable: Represents a table used to store lock state
//---------------------------------------------------------------------------
public class LMLockStateTable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public LMLockStateTable()
    {
	this.table = new Hashtable();
    }

    //-----------------------------------------------------------------------
    // Get the lock state associated with a given object ID
    //-----------------------------------------------------------------------
    public LMLockState get(String objectID)
    {
	LMLockState lockState = null;

	/*
	  NOTE: Don't check for the existence of files because we acquire
	  locks even before creating a non-existent file
	  f = new File(objectID);
	  if((!f.exists()) && (!objectID.equals(""))){
	  // NOTE: Empty string means global lock
	  Assert.affirm(false);
	  }
	*/
	lockState = (LMLockState)this.table.get(objectID);
	if(lockState == null){
	    lockState = new LMLockState(objectID);
	    this.table.put(objectID, lockState);
	}
	return(lockState);
    }

    //-----------------------------------------------------------------------
    // Remove lock state for a given object ID
    //-----------------------------------------------------------------------
    public void remove(String objectID)
    {
	this.table.remove(objectID);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	LMLockState ls1 = null;
	LMLockState ls2 = null;
	LMLockStateTable lst = null;

	lst = new LMLockStateTable();
	ls1 = lst.get("/");
	ls2 = lst.get("/");
	Assert.affirm(ls1 == ls2);
	ls1 = lst.get("");
	ls2 = lst.get("");
	Assert.affirm(ls1 == ls2);
	//lst.get("asdf");
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    private Hashtable table;
}
