import java.util.LinkedList;

//---------------------------------------------------------------------------
// SyncQueue: Implementation of a synchronized queue
//---------------------------------------------------------------------------
public class SyncQueue{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public SyncQueue()
    {
	this.localQueue = new LinkedList();
    }

    //-----------------------------------------------------------------------
    // Return true if the queue is empty
    //-----------------------------------------------------------------------
    public synchronized boolean isEmpty()
    {
	return(this.localQueue.isEmpty());
    }

    //-----------------------------------------------------------------------
    // Return the head of the queue; wait until one present
    //-----------------------------------------------------------------------
    public synchronized Object peekBlocked()
    {
	while(this.localQueue.isEmpty()){
	    try{
		this.wait();
	    }catch(InterruptedException e){
		System.err.println("" + e);
	    }
	}
	return(this.localQueue.getFirst());
    }

    //-----------------------------------------------------------------------
    // Remove the head of the queue. Queue may not be empty.
    //-----------------------------------------------------------------------
    public synchronized void dequeue()
    {
	Assert.affirm(!this.localQueue.isEmpty());
	this.localQueue.removeFirst();
    }

    //-----------------------------------------------------------------------
    // Insert an object at the end of the queue
    //-----------------------------------------------------------------------
    public synchronized void enqueue(Object obj)
    {
	this.localQueue.add(obj);
	this.notifyAll();
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	SyncQueue sq = null;
	Integer i = null;

	sq = new SyncQueue();
	new ProdThread(new Integer(3), 10, sq);
	new ProdThread(new Integer(4), 7, sq);
	new ConsThread(sq);
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    LinkedList localQueue;
}

//---------------------------------------------------------------------------
// ProdThread: A producer used for testing
//---------------------------------------------------------------------------
class ProdThread extends Thread{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public ProdThread(Object toInsert, int numTimes, SyncQueue queue)
    {
	this.toInsert = toInsert;
	this.numTimes = numTimes;
	this.queue = queue;
	this.start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	for(int i = 0; i < this.numTimes; i++){
	    try{
		Thread.sleep(1000);
	    }catch(InterruptedException e){
		System.err.println("" + e);
	    }
	    this.queue.enqueue(this.toInsert);
	}
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    Object toInsert;
    int numTimes;
    SyncQueue queue;
}

//---------------------------------------------------------------------------
// ConsThread: A consumer thread used for testing
//---------------------------------------------------------------------------
class ConsThread extends Thread{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public ConsThread(SyncQueue queue)
    {
	this.queue = queue;
	this.start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	Object obj = null;

	while(true){
	    obj = this.queue.peekBlocked();
	    System.out.println("" + obj);
	    this.queue.dequeue();
	}
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    SyncQueue queue;
}
