//---------------------------------------------------------------------------
// APMQReceiver: Acknowledged, persistent message queue receiver
//---------------------------------------------------------------------------
import java.io.IOException;

public class APMQReceiver{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public APMQReceiver(String sender, int portNum, int ackPortNum)
	throws IOException
    {
	this.receiver = new MQReceiver(sender, portNum);
	this.ackSender = new MQSender(sender, ackPortNum);
	this.seenSeqNum = -1;
	this.ackedSeqNum = -1;
    }

    //-----------------------------------------------------------------------
    // Dequeue the next message
    //-----------------------------------------------------------------------
    public OriginMsg dequeue()
    {
	OriginMsg msg = null;

	msg = this.receiver.dequeue();
	while(msg.getSeqNum() <= this.seenSeqNum){
	    msg = this.receiver.dequeue();
	}
	// NOTE: We might see jumps in seenSeqNum when the origin had an empty
	// queue and then restarted. So, this statement is incorrect:
	//assert((msg.getSeqNum() == (this.seenSeqNum + 1)) ||
	//	     (this.seenSeqNum == -1));
	assert((this.seenSeqNum == -1) || (msg.getSeqNum() > this.seenSeqNum));
	this.seenSeqNum = msg.getSeqNum();
	return(msg);
    }

    //-----------------------------------------------------------------------
    // Send an acknowledgment to the sender
    //-----------------------------------------------------------------------
    public void sendAck(long seqNum)
    {
	OriginMsg ackMsg = null;

	if((this.ackedSeqNum < 0) || (seqNum > this.ackedSeqNum)){
	    ackMsg = new OriginMsg(seqNum, AppMessage.ACK, null);
	    this.ackSender.enqueue(ackMsg);
	    ackMsg = null;
	    this.ackedSeqNum = seqNum;
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected MQReceiver receiver;
    protected MQSender ackSender;
    protected long seenSeqNum;
    protected long ackedSeqNum;
}
