//---------------------------------------------------------------------------
// APMQSendThread: Thread that continuously sends persistent messages
//---------------------------------------------------------------------------
import java.io.IOException;
import java.io.OutputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import java.net.Socket;
import java.net.InetAddress;

public class APMQSendThread implements Runnable{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public static final long INIT_TIMEOUT = 100;
    public static final long MAX_TIMEOUT = 10000;
    public static final double TIMEOUT_MULT_FACTOR = 2.0;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public APMQSendThread(PMsgStore msgStore, InetAddress address, int portNum)
	throws IOException
    {
	this.msgStore = msgStore;
	this.address = address;
	this.portNum = portNum;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	Socket socket = null;
	OutputStream os = null;
	byte[] dataBytes = null;
	DataOutputStream dataOS = null;
	ByteArrayOutputStream baos = null;
	OriginMsg msg = null;

	try{
	    while(true){
		socket = this.connect();
		System.out.println("Connected");
		try{
		    os = socket.getOutputStream();
		    baos = new ByteArrayOutputStream();
		    dataOS = new DataOutputStream(baos);
		    while(true){
			baos.reset();
			msg = (OriginMsg)this.msgStore.getNextMsg();
			msg.sendSelf(dataOS);
			dataOS.flush();
			dataBytes = baos.toByteArray();
			os.write(dataBytes);
			os.flush();
			//System.out.println("APMQSendThread: Sent message");
			Globals.stats.incRegBW(dataBytes.length);
			Globals.stats.incRegDataBW(msg);
		    }
		}catch(IOException e){
		    System.out.println("APMQSendThread: " + e);
		    if(dataOS != null){
			try{
			    dataOS.close();
			}catch(IOException e1){
			    // Do nothing
			}
		    }
		    this.closeSocket(socket);
		    this.msgStore.reset();
		}
	    }
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Keep trying to connect until successful
    //-----------------------------------------------------------------------
    private Socket connect()
    {
	long timeout = 0L;
	Socket socket = null;
	long wakeTime = 0L;
	boolean connected = false;

	timeout = APMQSendThread.INIT_TIMEOUT;
	connected = false;
	while(!connected){
	    try{
		socket = new Socket(this.address, this.portNum);
		socket.setTcpNoDelay(true);
		socket.setSendBufferSize(Globals.TCP_BUFFER_SIZE);
		System.out.println("Send buffer size = " +
				   socket.getSendBufferSize());
		connected = true;
	    }catch(IOException e){
		wakeTime = System.currentTimeMillis() + timeout;
		while(wakeTime > System.currentTimeMillis()){
		    try{
			Thread.sleep(wakeTime - System.currentTimeMillis());
		    }catch(InterruptedException ie){
			System.err.println("" + ie);
		    }
		}
		timeout = (long)(timeout * MQSendThread.TIMEOUT_MULT_FACTOR);
		if(timeout > APMQSendThread.MAX_TIMEOUT){
		    timeout = APMQSendThread.MAX_TIMEOUT;
		}
	    }
	}
	return(socket);
    }

    //-----------------------------------------------------------------------
    // Clean-up: Close all open sockets and streams
    //-----------------------------------------------------------------------
    private void closeSocket(Socket socket)
    {
	OutputStream os = null;

	assert(socket != null);
	try{
	    os = socket.getOutputStream();
	    os.close();
	}catch(IOException e){
	    // Do nothing
	}
	try{
	    socket.close();
	}catch(IOException e){
	    // Do nothing
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected PMsgStore msgStore;
    protected InetAddress address;
    protected int portNum;
}
