//---------------------------------------------------------------------------
// APMQSender: Acknowledged, persistent message queue sender
//---------------------------------------------------------------------------
import java.io.IOException;
import java.net.InetAddress;

public class APMQSender{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public APMQSender(String receiver, int portNum, int ackPortNum)
	throws IOException
    {
	InetAddress recvAddress = null;
	String dirName = null;

	recvAddress = InetAddress.getByName(receiver);
	dirName = APMQSender.makeDirName(recvAddress, portNum);
	this.msgBuffer = new PMsgStore(dirName);
	new APMQSendThread(this.msgBuffer, recvAddress, portNum);
	this.ackReceiver = new MQReceiver(receiver, ackPortNum);
	new AckHandleThread(this.ackReceiver, this.msgBuffer);
    }

    //-----------------------------------------------------------------------
    // Make a directory name from a port number and address
    //-----------------------------------------------------------------------
    private static String makeDirName(InetAddress recvAddress, int portNum)
    {
	return("q." + portNum + "." + recvAddress.getHostAddress());
    }

    //-----------------------------------------------------------------------
    // Enqueue the message to be sent
    //-----------------------------------------------------------------------
    public void enqueue(OriginMsg msg)
	throws ReplicaUnreceptiveException, IOException
    {
	this.msgBuffer.enqueue(msg);
    }

    //-----------------------------------------------------------------------
    // Get the highest sequence number seen
    //-----------------------------------------------------------------------
    public long getSeqNum()
    {
	return(this.msgBuffer.getMaxSeqNum());
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected PMsgStore msgBuffer;
    protected MQReceiver ackReceiver;
}
