//---------------------------------------------------------------------------
// AckHandleThread: Thread to handle incoming acknowledgments
//---------------------------------------------------------------------------
import java.io.IOException;

public class AckHandleThread implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public AckHandleThread(MQReceiver ackReceiver, PMsgStore msgStore)
    {
	this.ackReceiver = ackReceiver;
	this.msgStore = msgStore;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	long seqNum = 0;
	OriginMsg msg = null;

	try{
	    while(true){
		msg = this.ackReceiver.dequeue();
		assert(msg.getMessageType() == AppMessage.ACK);
		seqNum = msg.getSeqNum();
		msg = null;
		this.msgStore.gcUntil(seqNum);
	    }
	}catch(IOException e){
	    System.err.println("AckHandleThread: " + e);
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	AckHandleThread ackThread = null;
	MQReceiver receiver = null;
	PMsgStore msgStore = null;

	try{
	    receiver = new MQReceiver("localhost", 4566);
	    msgStore = new PMsgStore("a");
	    ackThread = new AckHandleThread(receiver, msgStore);

	    msgStore.enqueue(new OriginMsg(1000, AppMessage.READ, "dummyObj"));
	    msgStore.enqueue(new OriginMsg(1001, AppMessage.READ, "dummyObj"));

	    new AHTDummyThread(msgStore, 4566);
	}catch(IOException e){
	    System.err.println("" + e);
	    assert(false);
	}catch(ReplicaUnreceptiveException e){
	    System.err.println("" + e);
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected PMsgStore msgStore;
    protected MQReceiver ackReceiver;
}

class AHTDummyThread implements Runnable{
    public AHTDummyThread(PMsgStore msgStore, int portNum)
    {
	try{
	    this.msgStore = msgStore;
	    this.mqSender = new MQSender("localhost", portNum);
	    (new Thread(this)).start();
	}catch(IOException e){
	    System.err.println("" + e);
	    assert(false);
	}
    }

    public void run()
    {
	try{
	    System.out.println("Before first enqueue:");
	    //System.out.println("minSeqNum = " + msgStore.minSeqNum);
	    System.out.println("maxSeqNum = " + msgStore.getMaxSeqNum());
	    this.mqSender.enqueue(new OriginMsg(1000, AppMessage.ACK, ""));

	    Thread.sleep(1000);
	    System.out.println("After first enqueue:");
	    //System.out.println("minSeqNum = " + msgStore.minSeqNum);
	    System.out.println("maxSeqNum = " + msgStore.getMaxSeqNum());
	    this.mqSender.enqueue(new OriginMsg(1001, AppMessage.ACK, ""));

	    Thread.sleep(1000);
	    System.out.println("After second enqueue:");
	    //System.out.println("minSeqNum = " + msgStore.minSeqNum);
	    System.out.println("maxSeqNum = " + msgStore.getMaxSeqNum());
	}catch(InterruptedException e){
	    System.err.println("" + e);
	    assert(false);
	}
    }

    protected PMsgStore msgStore;
    protected MQSender mqSender;
}
