//---------------------------------------------------------------------------
// FSCommPacket: Represent the packet used to communicate with the FS
//---------------------------------------------------------------------------
import java.net.Socket;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;

public class FSCommPacket implements AppMessage{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FSCommPacket(int operation,
			int lockToken,
			String objectID,
			Socket socket)
    {
	this.operation = operation;
	this.lockToken = lockToken;
	this.objectID = objectID;
	this.replySocket = socket;
    }

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FSCommPacket(int operation,
			int lockToken,
			String objectID)
    {
	this.operation = operation;
	this.lockToken = lockToken;
	this.objectID = objectID;
	this.replySocket = null;
    }

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FSCommPacket(InputStream is, Socket replySocket) throws IOException
    {
	DataInputStream dis = null;
	int size = 0;
	byte[] objectIDBytes = null;

	dis = new DataInputStream(is);
	this.operation = dis.readInt();
	this.lockToken = dis.readInt();
	size = dis.readInt();
	objectIDBytes = new byte[size];
	dis.readFully(objectIDBytes);
	this.objectID = new String(objectIDBytes);
	this.replySocket = replySocket;
    }

    //-----------------------------------------------------------------------
    // Send this object over the output stream
    //-----------------------------------------------------------------------
    public void sendSelf(OutputStream os) throws IOException
    {
	byte[] dataBytes = null;
	DataOutputStream dos = null;
	byte[] objectIDBytes = null;
	ByteArrayOutputStream baos = null;

	try{
	    baos = new ByteArrayOutputStream(1000);
	    baos.reset();
	    dos = new DataOutputStream(baos);
	    dos.writeInt(this.operation);
	    dos.writeInt(this.lockToken);
	    objectIDBytes = this.objectID.getBytes();
	    dos.writeInt(objectIDBytes.length);
	    dos.write(objectIDBytes);
	    dos.flush();
	    dataBytes = baos.toByteArray();
	    os.write(dataBytes);
	    os.flush();
	}finally{
	    baos = null;
	    dos = null;
	    objectIDBytes = null;
	    dataBytes = null;
	}
    }

    //-----------------------------------------------------------------------
    // Return the sequence number (not applicable)
    //-----------------------------------------------------------------------
    public long getSeqNum()
    {
	assert(false);
	return(0L);
    }

    //-----------------------------------------------------------------------
    // Return the operation
    //-----------------------------------------------------------------------
    public int getMessageType()
    {
	return(this.operation);
    }

    //-----------------------------------------------------------------------
    // Return the object ID
    //-----------------------------------------------------------------------
    public String getObjectID()
    {
	return(this.objectID);
    }

    //-----------------------------------------------------------------------
    // Return true if this message has a lock (always true)
    //-----------------------------------------------------------------------
    public boolean containsLock()
    {
	return(true);
    }

    //-----------------------------------------------------------------------
    // Return the lock token
    //-----------------------------------------------------------------------
    public int getLockToken()
    {
	return(this.lockToken);
    }

    //-----------------------------------------------------------------------
    // Return data associated with this message (not applicable)
    //-----------------------------------------------------------------------
    public byte[] getData()
    {
	assert(false);
	return(null);
    }

    //-----------------------------------------------------------------------
    // Return the reply socket
    //-----------------------------------------------------------------------
    public Socket getReplySocket()
    {
	return(this.replySocket);
    }

    //-----------------------------------------------------------------------
    // Return the time stamp (not applicable)
    //-----------------------------------------------------------------------
    public long getTimeStamp()
    {
	assert(false);
	return(0L);
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int operation;
    protected int lockToken;
    protected String objectID;
    protected Socket replySocket;
}
