//---------------------------------------------------------------------------
// FileState: Used to store per-file state
//---------------------------------------------------------------------------
import java.net.Socket;

public class FileState{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public final static int STATE_IDLE = 0;
    public final static int STATE_GOT_DATA = 1;
    public final static int STATE_GOT_INV = 2;
    public final static int STATE_GOT_READ_AND_LOCK = 3;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FileState(String objectID)
    {
	this.buffer = null;
	this.stateName = STATE_IDLE;
	this.socket = null;
	this.objectID = objectID;
	this.seqNum = 0;
	this.lockToken = 0;
    }

    //-----------------------------------------------------------------------
    // Return the object ID
    //-----------------------------------------------------------------------
    public String getObjectID()
    {
	return(this.objectID);
    }

    //-----------------------------------------------------------------------
    // Return the current state
    //-----------------------------------------------------------------------
    public int getStateName()
    {
	return(this.stateName);
    }

    //-----------------------------------------------------------------------
    // Set the current state
    //-----------------------------------------------------------------------
    public void setStateName(int stateName)
    {
	this.stateName = stateName;

	// NOTE: Testing only
	switch(this.stateName){
	case STATE_IDLE:
	    Globals.debugPrintln("STATE_IDLE: " + this.getObjectID());
	    break;
	case STATE_GOT_DATA:
	    Globals.debugPrintln("STATE_GOT_DATA: " + this.getObjectID());
	    break;
	case STATE_GOT_INV:
	    Globals.debugPrintln("STATE_GOT_INV: " + this.getObjectID());
	    break;
	case STATE_GOT_READ_AND_LOCK:
	    Globals.debugPrintln("STATE_GOT_READ_AND_LOCK: " +
				 this.getObjectID());
	    break;
	default:
	    assert(false);
	    break;
	}
	Globals.debugPrintln("");
	// END
    }

    //-----------------------------------------------------------------------
    // Return the current buffer
    //-----------------------------------------------------------------------
    public byte[] getBuffer()
    {
	return(this.buffer);
    }

    //-----------------------------------------------------------------------
    // Fill the current buffer with the passed-in data
    //-----------------------------------------------------------------------
    public void fillBuffer(byte[] data)
    {
	this.buffer = data;
    }

    //-----------------------------------------------------------------------
    // Empty the current buffer
    //-----------------------------------------------------------------------
    public void emptyBuffer()
    {
	this.buffer = null;
    }

    //-----------------------------------------------------------------------
    // Set the socket which is waiting for a reply
    //-----------------------------------------------------------------------
    public void setReplySocket(Socket replySocket)
    {
	this.socket = replySocket;
    }

    //-----------------------------------------------------------------------
    // Return the socket that is waiting for a reply
    //-----------------------------------------------------------------------
    public Socket getReplySocket()
    {
	return(this.socket);
    }

    //-----------------------------------------------------------------------
    // Set the sequence number
    //-----------------------------------------------------------------------
    public void setSeqNum(long seqNum)
    {
	this.seqNum = seqNum;
    }

    //-----------------------------------------------------------------------
    // Return the sequence number
    //-----------------------------------------------------------------------
    public long getSeqNum()
    {
	return(this.seqNum);
    }

    //-----------------------------------------------------------------------
    // Set the lock token
    //-----------------------------------------------------------------------
    public void setLockToken(int lockToken)
    {
	this.lockToken = lockToken;
    }

    //-----------------------------------------------------------------------
    // Return the lock token
    //-----------------------------------------------------------------------
    public int getLockToken()
    {
	return(this.lockToken);
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected byte[] buffer;
    protected int stateName;
    protected Socket socket;
    protected String objectID;
    protected long seqNum;
    protected int lockToken;
}
