//---------------------------------------------------------------------------
// LMCommPoint: Used to communicate with the lock manager
//---------------------------------------------------------------------------
import java.net.Socket;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Hashtable;

public class LMCommPoint implements LMCommInterface{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public static final int LM_COMM_PORT = 5555;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public LMCommPoint()
    {
	try{
	    this.seqNum = 0;
	    this.mailBoxTable = new Hashtable();
	    this.socket = new Socket("localhost", LM_COMM_PORT);
	    this.socket.setTcpNoDelay(true);
	}catch(IOException e){
	    System.err.println("" + e);
	    assert(false);
	}
	new LMListener(this.socket, this.mailBoxTable);
    }

    //-----------------------------------------------------------------------
    // Send the request for a shared lock to the LM
    //-----------------------------------------------------------------------
    public synchronized void sendSharedLockRequest(String objectID,
						   LMMailBox mailBox)
    {
	LMRequestPacket request = null;
	OutputStream os = null;

	try{
	    this.seqNum++;
	    assert(!this.mailBoxTable.contains(new Long(this.seqNum)));
	    this.mailBoxTable.put(new Long(this.seqNum), mailBox);
	    request = new LMRequestPacket(AppMessage.LMOP_ACQUIRE_SHARED,
					  0,
					  this.seqNum,
					  objectID,
					  this.socket);
	    os = this.socket.getOutputStream();
	    request.sendSelf(os);
	    os.flush();
	}catch(IOException e){
	    System.err.println("LMCommPoint (requestSharedLock 1): " + e);
	    assert(false);
	}finally{
	    // Do not close these, just allow GC to decrement ref. counts
	    os = null;
	    request = null;
	}
    }

    //-----------------------------------------------------------------------
    // Request an exclusive lock on the passed-in object
    //-----------------------------------------------------------------------
    public synchronized void sendExclusiveLockRequest(String objectID,
						      LMMailBox mailBox)
    {
	LMRequestPacket request = null;
	OutputStream os = null;

	try{
	    this.seqNum++;
	    assert(!this.mailBoxTable.contains(new Long(this.seqNum)));
	    this.mailBoxTable.put(new Long(this.seqNum), mailBox);
	    request = new LMRequestPacket(AppMessage.LMOP_ACQUIRE_EXCLUSIVE,
					  0,
					  this.seqNum,
					  objectID,
					  this.socket);
	    os = this.socket.getOutputStream();
	    request.sendSelf(os);
	    os.flush();
	}catch(IOException e){
	    System.err.println("LMCommPoint (requestExclusveLock 1): " + e);
	    assert(false);
	}finally{
	    // Do not close these, just allow GC to decrement ref. counts
	    os = null;
	    request = null;
	}
    }

    //-----------------------------------------------------------------------
    // Release the passed-in lock
    //-----------------------------------------------------------------------
    public synchronized void releaseLock(String objectID, int lockToken)
    {
	LMRequestPacket request = null;
	OutputStream os = null;

	try{
	    request = new LMRequestPacket(AppMessage.LMOP_RELEASE,
					  lockToken,
					  this.seqNum,
					  objectID,
					  this.socket);
	    os = this.socket.getOutputStream();
	    request.sendSelf(os);
	    os.flush();
	}catch(IOException e){
	    System.err.println("LMCommPoint (releaseLock 1): " + e);
	    assert(false);
	}finally{
	    os = null;
	    request = null;
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int seqNum;
    protected Socket socket;
    protected Hashtable mailBoxTable;
}
