import java.util.Hashtable;
import java.net.InetAddress;

// For testing
import java.io.IOException;

//---------------------------------------------------------------------------
// MQQueuePool: Table of queues indexed by Internet addresses
//---------------------------------------------------------------------------
public class MQQueuePool{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public MQQueuePool()
    {
	this.table = new Hashtable();
    }

    //-----------------------------------------------------------------------
    // Return a queue for this address; make one if none present
    //-----------------------------------------------------------------------
    public synchronized SyncQueue getQueue(InetAddress address)
    {
	SyncQueue queue = null;

	queue = (SyncQueue)this.table.get(address);
	if(queue == null){
	    queue = new SyncQueue();
	    this.table.put(address, queue);
	}
	return(queue);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	InetAddress address = null;
	TestThread[] threadArray = null;
	SyncQueue queue = null;
	MQQueuePool pool = null;

	try{
	    address = InetAddress.getLocalHost();
	    pool = new MQQueuePool();
	    threadArray = new TestThread[10];
	    for(int i = 0; i < threadArray.length; i++){
		threadArray[i] = new TestThread(pool, address, i);
	    }
	    Thread.sleep(1000);
	    queue = pool.getQueue(address);
	    for(int i = 0; i < threadArray.length; i++){
		threadArray[i].join();
		assert(threadArray[i].getQueue() == queue);
	    }
	}catch(IOException e){
	    System.err.println("" + e);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    Hashtable table;
}

//---------------------------------------------------------------------------
// TestThread: Used to test the class
//---------------------------------------------------------------------------
class TestThread extends Thread{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public TestThread(MQQueuePool pool, InetAddress address, int threadNum)
    {
	this.pool = pool;
	this.address = address;
	this.threadNum = threadNum;
	this.queue = null;
	this.start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	long delay = 0;

	delay = (long)(Math.random() * 1000.0);
	try{
	    Thread.sleep(delay);
	    System.out.println("Thread " + this.threadNum + " started");
	    this.queue = this.pool.getQueue(address);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Return the queue
    //-----------------------------------------------------------------------
    public SyncQueue getQueue()
    {
	return(this.queue);
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    private MQQueuePool pool;
    private InetAddress address;
    private SyncQueue queue;
    private int threadNum;
}
