import java.util.Hashtable;
import java.net.InetAddress;

// For testing
import java.io.IOException;

//---------------------------------------------------------------------------
// MQRecvThreadPool: Maintain a pool of receive threads
//---------------------------------------------------------------------------
public class MQRecvThreadPool{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public MQRecvThreadPool(MQQueuePool queuePool)
    {
	this.queuePool = queuePool;
	this.table = new Hashtable();
    }

    //-----------------------------------------------------------------------
    // Get a thread from the pool indexed by IP address
    //-----------------------------------------------------------------------
    public synchronized MQRecvThread getThread(InetAddress address)
    {
	SyncQueue queue = null;
	MQRecvThread thread = null;

	thread = (MQRecvThread)this.table.get(address);
	if(thread == null){
	    queue = this.queuePool.getQueue(address);
	    thread = new MQRecvThread(queue);
	    this.table.put(address, thread);
	}
	return(thread);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
        InetAddress address = null;
        RPTestThread[] threadArray = null;
	MQRecvThread thread = null;
	MQRecvThreadPool recvThreadPool = null;

        try{
	    address = InetAddress.getLocalHost();
	    recvThreadPool = new MQRecvThreadPool(new MQQueuePool());
	    threadArray = new RPTestThread[10];
	    for(int i = 0; i < threadArray.length; i++){
		threadArray[i] = new RPTestThread(recvThreadPool, address, i);
	    }
	    Thread.sleep(1000);
	    thread = recvThreadPool.getThread(address);
	    for(int i = 0; i < threadArray.length; i++){
		threadArray[i].join();
		assert(threadArray[i].getThread() == thread);
	    }
	}catch(IOException e){
	    System.err.println("" + e);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    Hashtable table;
    MQQueuePool queuePool;
}

//---------------------------------------------------------------------------
// RPTestThread: Used to test the class
//---------------------------------------------------------------------------
class RPTestThread extends Thread{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public RPTestThread(MQRecvThreadPool pool,
			InetAddress address,
			int threadNum)
    {
        this.pool = pool;
        this.address = address;
        this.threadNum = threadNum;
        this.thread = null;
	this.start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
        long delay = 0;

        delay = (long)(Math.random() * 1000.0);
        try{
            Thread.sleep(delay);
            System.out.println("Thread " + this.threadNum + " started");
            this.thread = this.pool.getThread(address);
        }catch(InterruptedException e){
            System.err.println("" + e);
        }
    }

    //-----------------------------------------------------------------------
    // Return the thread
    //-----------------------------------------------------------------------
    public MQRecvThread getThread()
    {
        return(this.thread);
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    private MQRecvThreadPool pool;
    private InetAddress address;
    private MQRecvThread thread;
    private int threadNum;
}
