import java.net.Socket;
import java.net.ServerSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.io.InputStream;
import java.io.IOException;

//---------------------------------------------------------------------------
// MQServer: Implement the message queue listener
//---------------------------------------------------------------------------
public class MQServer implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public MQServer(MQQueuePool queuePool, int portNum) throws IOException
    {
	this.threadPool = new MQRecvThreadPool(queuePool);
	this.serverSocket = new ServerSocket();
	this.serverSocket.setReceiveBufferSize(Globals.TCP_BUFFER_SIZE);
	this.serverSocket.bind(new InetSocketAddress(portNum));
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	Socket socket = null;
	InetAddress address = null;
	MQRecvThread recvThread = null;

	try{
	    while(true){
		socket = null;
		try{
		    socket = this.serverSocket.accept();
		    socket.setTcpNoDelay(true);
		    socket.setReceiveBufferSize(Globals.TCP_BUFFER_SIZE);
		    address = socket.getInetAddress();
		    recvThread = this.threadPool.getThread(address);
		    recvThread.stopThread();
		    try{
			recvThread.startThread(socket);
		    }catch(IOException e){
			recvThread.stopThread();
		    }
		}catch(IOException e){
		    if(socket != null){
			this.closeSocket(socket);
		    }
		}
	    }
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Clean-up work; close the socket and associated input streams
    //-----------------------------------------------------------------------
    private void closeSocket(Socket socket)
    {
	InputStream is = null;

	try{
	    is = socket.getInputStream();
	    is.close();
	}catch(IOException e){
	    System.err.println("MQServer (closeSocket 1): " + e);
	}
	try{
	    socket.close();
	}catch(IOException e){
	    System.err.println("MQServer (closeSocket 2): " + e);
	}
    }

    //-----------------------------------------------------------------------
    // Private data
    //-----------------------------------------------------------------------
    ServerSocket serverSocket;
    MQRecvThreadPool threadPool;
}
