//---------------------------------------------------------------------------
// Implement a SocketImpl instance for a NiceSocket
//---------------------------------------------------------------------------
#include <jni.h>
#include "NiceSocketImpl.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <sys/types.h>

#define SEND_BUF_SIZE 5000000

//---------------------------------------------------------------------------
// Create a socket
//---------------------------------------------------------------------------
JNIEXPORT jint JNICALL Java_NiceSocketImpl_c_1socket(JNIEnv *env, jobject obj)
{
  int size = 0;
  jint sockFD = 0;
  int sendBufSize = 0;
  int reply = 0;
  int make_it_one = 1;

  sockFD = socket(AF_INET, SOCK_STREAM, 0);

  if(sockFD >= 0){
    // See if this keeps Nice from stealing bw
    reply = setsockopt(sockFD,
                       SOL_TCP,
                       TCP_NODELAY,
                       (char*)(&make_it_one),
                       (socklen_t)sizeof(int));
  }

  if(sockFD >= 0){
    // Set the receiver buffer size
    sendBufSize = SEND_BUF_SIZE;
    setsockopt(sockFD, SOL_SOCKET, SO_SNDBUF, &sendBufSize, sizeof(int));
    // See if this worked
    size = sizeof(sendBufSize);
    getsockopt(sockFD, SOL_SOCKET, SO_SNDBUF, &sendBufSize, &size);
    printf("sendBufSize = %d, size = %d\n", sendBufSize, size);
  }

  if(sockFD >= 0){
    setsockopt(sockFD, IPPROTO_TCP, 14, &make_it_one, sizeof(int));
    // See if this worked
    size = sizeof(make_it_one);
    getsockopt(sockFD, IPPROTO_TCP, 14, &make_it_one, &size);
    printf("make_it_one = %d, size = %d\n", make_it_one, size);
  }

  return(sockFD);
}

//---------------------------------------------------------------------------
// Connect a socket
//---------------------------------------------------------------------------
JNIEXPORT jint JNICALL Java_NiceSocketImpl_c_1connect(JNIEnv *env,
						      jobject obj,
						      jint sock,
						      jstring ipAddress,
						      jint port)
{
  jint retVal = -1;
  struct sockaddr_in server;
  struct hostent *hp;
  const char* hostIPAddress = NULL;

  hostIPAddress = (*env)->GetStringUTFChars(env, ipAddress, 0);
  hp = gethostbyname(hostIPAddress);
  // If this seg faults, put this statement at the end
  (*env)->ReleaseStringUTFChars(env, ipAddress, hostIPAddress);

  if(hp != NULL){
    server.sin_family = AF_INET;
    server.sin_port = htons((short)port);
    memcpy((char*)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
    retVal = connect(sock, (struct sockaddr *)&server, sizeof(server));
  }

  return(retVal);
}

//---------------------------------------------------------------------------
// Close a socket
//---------------------------------------------------------------------------
JNIEXPORT jint JNICALL Java_NiceSocketImpl_c_1close(JNIEnv *env,
						    jobject obj,
						    jint sock)
{
  jint retVal = 0;
  retVal = close(sock);
  return(retVal);

}
