//---------------------------------------------------------------------------
// Implementation of a TCP Nice socket
//---------------------------------------------------------------------------
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.FileDescriptor;
import java.net.Socket;
import java.net.SocketImpl;
import java.net.InetAddress;

// For Java 1.4 compatibility
import java.net.SocketAddress;

public class NiceSocketImpl extends SocketImpl{

    //-----------------------------------------------------------------------
    // JNI stuff
    //-----------------------------------------------------------------------
    static{
	System.loadLibrary("nicesock");
    }

    public native int c_socket();
    public native int c_connect(int sock, String ipAddress, int port);
    public native int c_close(int sock);

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public NiceSocketImpl()
    {
	this.sockFD = 0;
	this.is = null;
	this.os = null;
    }

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public NiceSocketImpl(int sockFD, InetAddress remoteAddress)
    {
	this.sockFD = sockFD;
	this.remoteAddress = remoteAddress;
	this.is = new NiceSocketInputStream(this.sockFD);
	this.os = new NiceSocketOutputStream(this.sockFD);
    }

    //-----------------------------------------------------------------------
    // Accept a connection
    //-----------------------------------------------------------------------
    protected void accept(SocketImpl s)
    {
	Globals.debugPrintln("NiceSocketImpl.accept()");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Returns the number of bytes that can be read from this socket
    // without blocking.
    //-----------------------------------------------------------------------
    protected int available() throws IOException
    {
	Globals.debugPrintln("NiceSocketImpl.available()");
	assert(false);
	return(0);
    }

    //-----------------------------------------------------------------------
    // Binds this socket to the specified port number on the specified host.
    //-----------------------------------------------------------------------
    protected void bind(InetAddress host, int port)
    {
	Globals.debugPrintln("NiceSocketImpl.bind(" + host +
			     ", " + port + ")");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Closes this socket.
    //-----------------------------------------------------------------------
    public void close() throws IOException
    {
	int retVal = 0;

	Globals.debugPrintln("NiceSocketImpl.close()");
	retVal = c_close(this.sockFD);
	if(retVal < 0){
	    throw(new IOException("Error closing socket"));
	}
    }

    //-----------------------------------------------------------------------
    // Connects this socket to the specified port number on the
    // specified host.
    //-----------------------------------------------------------------------
    public void connect(InetAddress address, int port) throws IOException
    {
	String ipAddress = null;
	int retVal = 0;

	Globals.debugPrintln("NiceSocketImpl.connect1(" +
			     address + ", " + port + ")");
	ipAddress = address.getHostAddress();
	retVal = c_connect(this.sockFD, ipAddress, port);
	if(retVal < 0){
	    throw(new IOException("Could not connect socket"));
	}
	this.remoteAddress = address;
	this.is = new NiceSocketInputStream(this.sockFD);
	this.os = new NiceSocketOutputStream(this.sockFD);
    }

    //-----------------------------------------------------------------------
    // Connects this socket to the specified port on the named host.
    //-----------------------------------------------------------------------
    public void connect(String host, int port) throws IOException
    {
	InetAddress inetAddress = null;
	String ipAddress = null;
	int retVal = 0;

	inetAddress = InetAddress.getByName(host);
	ipAddress = inetAddress.getHostAddress();
	Globals.debugPrintln("NiceSocketImpl.connect2(" + ipAddress + ", " +
			     port + ")");
	retVal = c_connect(this.sockFD, ipAddress, port);
	if(retVal < 0){
	    throw(new IOException("Could not connect socket"));
	}
	this.remoteAddress = inetAddress;
	this.is = new NiceSocketInputStream(this.sockFD);
	this.os = new NiceSocketOutputStream(this.sockFD);
    }

    //-----------------------------------------------------------------------
    // Creates either a stream or a datagram socket.
    //-----------------------------------------------------------------------
    public void create(boolean stream) throws IOException
    {
	Globals.debugPrintln("NiceSocketImpl.create(" + stream + ")");
	assert(stream);
	this.sockFD = c_socket();
	if(this.sockFD < 0){
	    throw(new IOException("Could not create socket"));
	}
	if(Globals.VERBOSE){
	    System.out.println("socket fd = " + sockFD);
	}
    }

    //-----------------------------------------------------------------------
    // Returns the value of this socket's fd field.
    //-----------------------------------------------------------------------
    protected FileDescriptor getFileDescriptor()
    {
	Globals.debugPrintln("NiceSocketImpl.getFileDescriptor()");
	assert(false);
	return(null);
    }

    //-----------------------------------------------------------------------
    // Returns the value of this socket's address field.
    //-----------------------------------------------------------------------
    protected InetAddress getInetAddress()
    {
	Globals.debugPrintln("NiceSocketImpl.getInetAddress()");
	return(this.remoteAddress);
    }

    //-----------------------------------------------------------------------
    // Returns an input stream for this socket.
    //-----------------------------------------------------------------------
    protected InputStream getInputStream()
    {
	Globals.debugPrintln("NiceSocketImpl.getInputStream()");
	return(this.is);
    }

    //-----------------------------------------------------------------------
    // Returns the value of this socket's localport field.
    //-----------------------------------------------------------------------
    protected int getLocalPort()
    {
	Globals.debugPrintln("NiceSocketImpl.getLocalPort()");
	assert(false);
	return(0);
    }

    //-----------------------------------------------------------------------
    // Returns an output stream for this socket.
    //-----------------------------------------------------------------------
    protected OutputStream getOutputStream()
    {
	Globals.debugPrintln("NiceSocketImpl.getOutputStream()");
	return(this.os);
    }

    //-----------------------------------------------------------------------
    // Returns the value of this socket's port field.
    //-----------------------------------------------------------------------
    protected int getPort()
    {
	Globals.debugPrintln("NiceSocketImpl.getPort()");
	assert(false);
	return(0);
    }

    //-----------------------------------------------------------------------
    // Sets the maximum queue length for incoming connection indications
    // (a request to connect) to the count argument.
    //-----------------------------------------------------------------------
    protected void listen(int backlog)
    {
	Globals.debugPrintln("NiceSocketImpl.listen(" + backlog + ")");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Places the input stream for this socket at "end of stream".
    //-----------------------------------------------------------------------
    protected void shutdownInput()
    {
	Globals.debugPrintln("NiceSocketImpl.shutdownInput()");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Disables the output stream for this socket.
    //-----------------------------------------------------------------------
    protected void shutdownOutput()
    {
	Globals.debugPrintln("NiceSocketImpl.shutdownOutput()");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Returns the address and port of this socket as a String.
    //-----------------------------------------------------------------------
    public String toString()
    {
	Globals.debugPrintln("NiceSocketImpl.toString()");
	assert(false);
	return(null);
    }

    //-----------------------------------------------------------------------
    // FROM SocketOptions:
    //-----------------------------------------------------------------------

    //-----------------------------------------------------------------------
    // Fetch the value of an option.
    //-----------------------------------------------------------------------
    public Object getOption(int optID)
    {
	Globals.debugPrintln("NiceSocketImpl.getOption(" + optID + ")");
	assert(false);
	return(null);
    }

    //-----------------------------------------------------------------------
    // Enable/disable the option specified by optID.
    //-----------------------------------------------------------------------
    public void setOption(int optID, Object value)
    {
	Globals.debugPrintln("NiceSocketImpl.setOption(" + optID +
			     ", " + value + ")");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // FROM Java 1.4: (Comment when compiling with Java 1.3)
    //-----------------------------------------------------------------------

    //-----------------------------------------------------------------------
    // Connects this socket to the specified port on the named host.
    //-----------------------------------------------------------------------
    protected void connect(SocketAddress address, int timeout)
	throws IOException
    {
	Globals.debugPrintln("NiceSocketImpl.connect3(" + address + ", " +
			     timeout + ")");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Send one byte of urgent data on the socket
    //-----------------------------------------------------------------------
    protected void sendUrgentData(int data)
    {
	Globals.debugPrintln("NiceSocketImpl.sendUrgentData(" + data + ")");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int sockFD;
    protected InputStream is;
    protected OutputStream os;
    protected InetAddress remoteAddress;
}
