//---------------------------------------------------------------------------
// Implementation of a TCP Nice socket input stream
//---------------------------------------------------------------------------
import java.io.InputStream;
import java.io.IOException;

public class NiceSocketInputStream extends InputStream
{
    //-----------------------------------------------------------------------
    // JNI stuff
    //-----------------------------------------------------------------------
    static{
	System.loadLibrary("nicesockis");
    }

    public native int c_recv(int sockFD, byte[] b, int off, int len);
    public native int c_close(int sockFD);

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public NiceSocketInputStream(int sockFD)
    {
	Globals.debugPrintln("NiceSocketInputStream.NiceSocketInputStream(" +
			     sockFD + ")");
	this.sockFD = sockFD;
    }

    //-----------------------------------------------------------------------
    // Returns the number of bytes that can be read (or skipped over) from
    // this input stream without blocking by the next caller of a method for
    // this input stream.
    //-----------------------------------------------------------------------
    public int available()
    {
	Globals.debugPrintln("NiceSocketInputStream.available()");
	assert(false);
	return(0);
    }

    //-----------------------------------------------------------------------
    // Closes this input stream and releases any system resources associated
    // with the stream.
    //-----------------------------------------------------------------------
    public void close() throws IOException
    {
	int retVal = 0;

	Globals.debugPrintln("NiceSocketInputStream.close()");
	retVal = c_close(this.sockFD);
	if(retVal < 0){
	    throw(new IOException("Error closing socket"));
	}
    }

    //-----------------------------------------------------------------------
    // Marks the current position in this input stream.
    //-----------------------------------------------------------------------
    public void mark(int readlimit)
    {
	Globals.debugPrintln("NiceSocketInputStream.mark() not implemented!");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Tests if this input stream supports the mark and reset methods.
    //-----------------------------------------------------------------------
    public boolean markSupported()
    {
	Globals.debugPrintln("NiceSocketInputStream.markSupported()");
	return(false);
    }

    //-----------------------------------------------------------------------
    // Reads the next byte of data from the input stream.
    //-----------------------------------------------------------------------
    public int read()
    {
	Globals.debugPrintln("NiceSocketInputStream.read() not implemented!");
	assert(false);
	return(0);
    }

    //-----------------------------------------------------------------------
    // Reads some number of bytes from the input stream and stores them into
    // the buffer array b.
    //-----------------------------------------------------------------------
    public int read(byte[] b) throws IOException
    {
	int retVal = 0;

	Globals.debugPrintln("NiceSocketInputStream.read(byte[])");
	retVal = c_recv(this.sockFD, b, 0, b.length);
	if(retVal < 0){
	    throw(new IOException("Error writing to socket"));
	}
	return(retVal);
    }

    //-----------------------------------------------------------------------
    // Reads up to len bytes of data from the input stream into an array
    // of bytes.
    //-----------------------------------------------------------------------
    public int read(byte[] b, int off, int len) throws IOException
    {
	int retVal = 0;

	Globals.debugPrintln("NiceSocketInputStream.read(byte[], int, int)");
	retVal = c_recv(this.sockFD, b, off, len);
	if(retVal < 0){
	    throw(new IOException("Error writing to socket"));
	}
	return(retVal);
    }

    //-----------------------------------------------------------------------
    // Repositions this stream to the position at the time the
    // mark method was last called on this input stream.
    //-----------------------------------------------------------------------
    public void reset()
    {
	Globals.debugPrintln("NiceSocketInputStream.reset() not implemented!");
	assert(false);
    }

    //-----------------------------------------------------------------------
    // Skips over and discards n bytes of data from this input stream.
    //-----------------------------------------------------------------------
    public long skip(long n)
    {
	Globals.debugPrintln("NiceSocketInputStream.skip() not implemented!");
	assert(false);
	return(0);
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int sockFD;
}
