//---------------------------------------------------------------------------
// NiceThread: Implementation of the thread that receives prefetched data
//---------------------------------------------------------------------------
import java.net.Socket;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.IOException;

public class NiceThread extends AtomicThread{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public NiceThread(OriginQueueFSM fsm,
		      PrefetchClient prefetchChannel,
		      BooleanLock lock)
    {
	super(lock);
	this.fsm = fsm;
	this.prefetchChannel = prefetchChannel;
	this.startSelf();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	OriginMsg msg = null;
	InputStream is = null;
	DataInputStream dataIS = null;

	try{
	    while(true){
		this.waitUntilActive();
		while(true){
		    msg = this.prefetchChannel.dequeue();
		    assert(msg.getMessageType() == OriginMsg.DATA);
		    assert(msg.getData() != null);
		    /*
		      System.out.println("Received message of type " +
		      msg.getMessageType() +
		      " on the prefetch channel");
		    */
		    this.enterCriticalSection();
		    this.fsm.handlePrefData(msg);
		    assert(msg.getData() != null);
		    msg = null;
		    this.exitCriticalSection();
		}
	    }
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected OriginQueueFSM fsm;
    protected PrefetchClient prefetchChannel;
}
