//---------------------------------------------------------------------------
// OldDataPool: Contains unaccepted old data messages
//
// NOTES: Currently dequeuePriority() only returns some enqueued message.
//        Won't change unless the replica can move too far ahead of the
//        message layer.
//---------------------------------------------------------------------------
import java.util.Hashtable;
import java.util.Enumeration;

public class OldDataPool{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public OldDataPool()
    {
	this.table = new Hashtable();
	this.totalSize = 0;
    }

    //-----------------------------------------------------------------------
    // Enqueue a message into the pool
    //-----------------------------------------------------------------------
    public synchronized void enqueue(OriginMsg dataMsg)
    {
	OriginMsg oldMsg = null;

	assert(dataMsg.getMessageType() == AppMessage.DATA);
	oldMsg = (OriginMsg)this.table.get(dataMsg.getObjectID());
	if((oldMsg == null) || (oldMsg.getSeqNum() <= dataMsg.getSeqNum())){
	    this.table.put(dataMsg.getObjectID(), dataMsg);
	    this.totalSize += (dataMsg.getData()).length;
	    // We shouldn't be using more than 40MB behind
	    assert(this.totalSize < 40000000);
	    /*
	      System.out.println("OldDataPool: this.totalSize = " +
	      this.totalSize);
	    */
	    this.notifyAll();
	}
    }

    //-----------------------------------------------------------------------
    // Dequeue the highest priority message
    // NOTE: Does not quite do this currently
    //-----------------------------------------------------------------------
    public synchronized OriginMsg dequeuePriority()
    {
	Enumeration e = null;
	String objectID = null;
	OriginMsg dataMsg = null;

	e = this.table.keys();
	assert(e.hasMoreElements());
	objectID = (String)e.nextElement();
	dataMsg = (OriginMsg)this.table.remove(objectID);
	assert(dataMsg != null);
	this.totalSize -= (dataMsg.getData()).length;
	assert(this.totalSize >= 0);
	return(dataMsg);
    }

    //-----------------------------------------------------------------------
    // Dequeue the data message with the given object ID
    //-----------------------------------------------------------------------
    public synchronized OriginMsg dequeue(String objectID)
    {
	OriginMsg msg = null;
	msg = (OriginMsg)this.table.remove(objectID);
	if(msg != null){
	    this.totalSize -= (msg.getData()).length;
	    assert(this.totalSize >= 0);
	}
	return(msg);
    }

    //-----------------------------------------------------------------------
    // Return true if the data pool is empty
    //-----------------------------------------------------------------------
    public synchronized boolean isEmpty()
    {
	return(this.table.isEmpty());
    }

    //-----------------------------------------------------------------------
    // Remove any older version of data for the same object ID
    //-----------------------------------------------------------------------
    public synchronized void removeOlderVersion(OriginMsg msg)
    {
	String objectID = null;
	OriginMsg oldMsg = null;

	objectID = msg.getObjectID();
	oldMsg = (OriginMsg)this.table.remove(objectID);
	if(oldMsg != null){	
	    assert(oldMsg.getSeqNum() < msg.getSeqNum());
	}
    }

    //-----------------------------------------------------------------------
    // Wait for data to arrive
    //-----------------------------------------------------------------------
    public synchronized void waitForData()
    {
	while(this.table.isEmpty()){
	    try{
		this.wait();
	    }catch(InterruptedException e){
		System.err.println("" + e);
	    }
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected long totalSize;
    protected Hashtable table;
}
