//---------------------------------------------------------------------------
// Origin: Represents the origin in the system
//---------------------------------------------------------------------------
import java.io.IOException;
import java.net.InetAddress;

public class Origin{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public static final int DEMAND_ONLY_ALGO = 1;
    public static final int UPDATE_ALL_ALGO = 2;
    public static final int PREF_INVAL_ALGO = 3;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public Origin(int algorithm,
		  SyncLMCommPoint lmComm,
		  String[] replicaIDs,
		  int regPort,
		  int nicePort,
		  int ackPort)
    {
	InetAddress replicaAddr = null;
	PriorityQueue priorityQueue = null;
	ReplicaManager replicaManager = null;
	PrioritySendThread prefetchThread = null;

	try{
	    this.algorithm = algorithm;
	    this.replicaManagers = new ReplicaManager[replicaIDs.length];
	    this.prefetchServer = new PrefetchServer(nicePort);

	    for(int i = 0; i < replicaIDs.length; i++){
		priorityQueue = new PriorityQueue();
		replicaManager = new ReplicaManager(replicaIDs[i],
						    regPort,
						    nicePort,
						    ackPort,
						    priorityQueue);
		this.replicaManagers[i] = replicaManager;
		prefetchThread = new PrioritySendThread(priorityQueue, lmComm);
		replicaAddr = InetAddress.getByName(replicaIDs[i]);
		this.prefetchServer.addBinding(replicaAddr, prefetchThread);
		new ReplicaListener(this.replicaManagers[i],
				    replicaIDs[i],
				    regPort,
				    ackPort,
				    lmComm);
	    }
	}catch(IOException e){
	    System.err.println("Origin (Origin 1) " + e);
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public void replicateData(String objectID, int lockToken, double priority)
    {
	for(int i = 0; i < this.replicaManagers.length; i++){
	    switch(this.algorithm){
	    case DEMAND_ONLY_ALGO:
		this.replicaManagers[i].enqueueInval(objectID,
						     lockToken,
						     priority);
		break;
	    case UPDATE_ALL_ALGO:
		this.replicaManagers[i].enqueueDemandReply(objectID,
							   lockToken);
		break;
	    case PREF_INVAL_ALGO:
		this.replicaManagers[i].enqueueUpdate(objectID,
						      lockToken,
						      priority);
		break;
	    default:
		assert(false);
		break;
	    }
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int algorithm;
    protected PrefetchServer prefetchServer;
    protected ReplicaManager[] replicaManagers;
}
