//---------------------------------------------------------------------------
// OriginMsgQueue: Handles delaying invals and synchronizing updates
//---------------------------------------------------------------------------
import java.io.IOException;

public class OriginMsgQueue{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public OriginMsgQueue(ReplicaFSM replicaFSM,
			  String serverName,
			  int regPort,
			  int ackPort,
			  int nicePort,
			  long timeout,
			  BooleanLock syncLock) throws IOException
    {
	SyncQueue localQueue = null;
	APMQSender demReqChannel = null;
	PrefetchClient prefChannel = null;

	localQueue = new SyncQueue();
	demReqChannel = new APMQSender(serverName, regPort, ackPort);
	prefChannel = new PrefetchClient(serverName, nicePort);
	this.syncLock = syncLock;
	this.demInvChannel = new APMQReceiver(serverName, regPort, ackPort);
	this.originQueueFSM = new OriginQueueFSM(syncLock,
						 localQueue,
						 timeout,
						 demReqChannel,
						 this.demInvChannel,
						 prefChannel);
	new OriginMsgApplyThread(replicaFSM, localQueue);
    }

    //-----------------------------------------------------------------------
    // Send a request for an object to the origin server
    //-----------------------------------------------------------------------
    public void sendObjectRequest(String objectID, long seqNum)
    {
	AtomicThread.acquireCSLock(this.syncLock);
	this.originQueueFSM.startRollMode(objectID, seqNum);
	AtomicThread.releaseCSLock(this.syncLock);
    }

    //-----------------------------------------------------------------------
    // Send an acknowledgment for a message
    //-----------------------------------------------------------------------
    public void sendAck(long seqNum)
    {
	// Don't need to be in the critical section for this
	this.demInvChannel.sendAck(seqNum);
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected OriginQueueFSM originQueueFSM;
    protected BooleanLock syncLock;
    protected APMQReceiver demInvChannel;
}
