//---------------------------------------------------------------------------
// OriginTest: Tool to test the origin
//---------------------------------------------------------------------------
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;

public class OriginTest{

    //-----------------------------------------------------------------------
    // Program starting point
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	// Read a file name from stdin and distribute it
	Reader reader = new BufferedReader(new InputStreamReader(System.in));
	int token = 0;
	LineNumberReader lnr = null;
	String line = null;
	int spIndex = 0;
	double priority = 0.0;

	OriginTest.parseArgs(argv);
	OriginTest.lmComm = new SyncLMCommPoint(new LMCommPoint());
	OriginTest.origin = new Origin(OriginTest.algorithm,
				       OriginTest.lmComm,
				       OriginTest.replicaNames,
				       OriginTest.regPort,
				       OriginTest.nicePort,
				       OriginTest.ackPort);

	try{
	    lnr = new LineNumberReader(reader);
	    line = lnr.readLine();
	    while(line != null){
		spIndex = line.indexOf(' ');
		if(spIndex >= 0){
		    try{
			priority = Double.parseDouble(line.substring(spIndex +
								     1));
			OriginTest.makeRequest(line.substring(0, spIndex),
					       priority);
		    }catch(NumberFormatException e){
			System.err.println("" + e);
		    }
		}
		line = lnr.readLine();
	    }
	}catch(IOException e){
	    System.err.println("OriginTest (main 1): " + e);
	}
    }

    //-----------------------------------------------------------------------
    // Make the actual request
    //-----------------------------------------------------------------------
    protected static void makeRequest(String objectID, double priority)
    {
	int lockToken = 0;
	AppMessage lockMsg = null;

	System.out.println("objectID = " + objectID);
	System.out.println("priority = " + priority);

	lockMsg = OriginTest.lmComm.requestExclusiveLock(objectID);
	lockToken = lockMsg.getLockToken();
	OriginTest.origin.replicateData(objectID, lockToken, priority);
	OriginTest.lmComm.releaseLock(objectID, lockToken);
    }

    //-----------------------------------------------------------------------
    // Get the replica names from the command line
    //-----------------------------------------------------------------------
    protected static void parseArgs(String[] argv)
    {
	if(argv.length >= 5){
	    OriginTest.replicaNames = new String[argv.length - 4];
	    System.arraycopy(argv,
			     0,
			     OriginTest.replicaNames,
			     0,
			     argv.length - 4);
	    OriginTest.algorithm = Integer.parseInt(argv[argv.length - 4]);
	    OriginTest.regPort = Integer.parseInt(argv[argv.length - 3]);
	    OriginTest.nicePort = Integer.parseInt(argv[argv.length - 2]);
	    OriginTest.ackPort = Integer.parseInt(argv[argv.length - 1]);
	}else{
	    System.out.println("Usage: OriginTest <replicaname>+ <algorithm>" +
			       " <regport> <niceport> <ackport>");
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected static String[] replicaNames = null;
    protected static int algorithm = 0;
    protected static int regPort = 0;
    protected static int nicePort = 0;
    protected static int ackPort = 0;
    protected static Origin origin = null;
    protected static SyncLMCommPoint lmComm = null;
}
