//---------------------------------------------------------------------------
// PMaxSeqNumFile: Periodically store a sequence number to restart from
//---------------------------------------------------------------------------
import java.io.IOException;
import java.io.EOFException;
import java.io.RandomAccessFile;

public class PMaxSeqNumFile{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public static final int SEQ_NUM_STRIDE = 1000;

    // FOR TESTING ONLY!:
    //public static final int SEQ_NUM_STRIDE = 5;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PMaxSeqNumFile(String fileName) throws IOException
    {
	this.raf = new RandomAccessFile(fileName, "rw");
	try{
	    this.lastKnownSeqNum = this.raf.readLong();
	}catch(EOFException e){
	    this.lastKnownSeqNum = 0;
	    this.raf.writeLong(this.lastKnownSeqNum);
	}
    }

    //-----------------------------------------------------------------------
    // Return the next usable sequence number
    //-----------------------------------------------------------------------
    public long getNextSafeSeqNum()
    {
	return(this.lastKnownSeqNum + SEQ_NUM_STRIDE);
    }

    //-----------------------------------------------------------------------
    // Update the sequence number
    //-----------------------------------------------------------------------
    public void update(long seqNum) throws IOException
    {
	// This is wrong because after a restart sequence numbers may not
	// follow where they left off
	//assert((seqNum == (this.seqNum + 1)) || (seqNum < this.seqNum));

	// Make sure we don't go back in time
	Assert.affirm(seqNum > this.lastKnownSeqNum);
	if(seqNum >= (this.lastKnownSeqNum + SEQ_NUM_STRIDE)){
	    this.raf.seek(0);
	    this.raf.writeLong(seqNum);
	    this.lastKnownSeqNum = seqNum;
	}
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	PMaxSeqNumFile f = null;
	long s = 0;

	try{
	    f = new PMaxSeqNumFile(argv[0]);
	    s = f.getNextSafeSeqNum();
	    System.out.println("seqnum = " + f.getNextSafeSeqNum());
	    f.update(s + 4);
	    System.out.println("seqnum = " + f.getNextSafeSeqNum());
	    f.update(s + 8);
	    System.out.println("seqnum = " + f.getNextSafeSeqNum());
	}catch(IOException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected RandomAccessFile raf;
    protected long lastKnownSeqNum;
}
