//---------------------------------------------------------------------------
// PrefDataPool: Pool of data messages not ready to be accepted
//---------------------------------------------------------------------------
import java.util.Hashtable;
import java.util.LinkedList;

public class PrefDataPool{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PrefDataPool()
    {
	this.objectIndexTable = new Hashtable();
	this.seqNumIndexTable = new Hashtable();
	this.totalSize = 0;
    }

    //-----------------------------------------------------------------------
    // Dequeue the message with the given sequence number
    //-----------------------------------------------------------------------
    public OriginMsg dequeue(long seqNum)
    {
	OriginMsg dataMsg = null;
	String objectID = null;
	LinkedList objectList = null;
	boolean success = false;

	dataMsg = (OriginMsg)this.seqNumIndexTable.remove(new Long(seqNum));
	if(dataMsg != null){
	    objectID = dataMsg.getObjectID();
	    objectList = (LinkedList)this.objectIndexTable.get(objectID);
	    assert(objectList != null);
	    success = objectList.remove(dataMsg);
	    assert(success);
	    this.totalSize -= (dataMsg.getData()).length;
	    assert(this.totalSize >= 0);
	    //System.out.println("PrefDataPool: Size = " + this.totalSize);
	}
	return(dataMsg);
    }

    //-----------------------------------------------------------------------
    // Enqueue the given message
    //-----------------------------------------------------------------------
    public void enqueue(OriginMsg dataMsg)
    {
	String objectID = null;
	LinkedList objectList = null;
	OriginMsg lastMsg = null;

	this.seqNumIndexTable.put(new Long(dataMsg.getSeqNum()), dataMsg);
	objectID = dataMsg.getObjectID();
	objectList = (LinkedList)this.objectIndexTable.get(objectID);
	if(objectList == null){
	    objectList = new LinkedList();
	    this.objectIndexTable.put(objectID, objectList);
	}
	// Next "if" can be removed; for testing only
	if(!objectList.isEmpty()){
	    lastMsg = (OriginMsg)objectList.getLast();
	    assert(lastMsg.getSeqNum() < dataMsg.getSeqNum());
	}
	objectList.addLast(dataMsg);
	this.totalSize += (dataMsg.getData()).length;
	// Intentionally limit the pref buffer size to 40MB
	assert(this.totalSize <= 40000000);
	//System.out.println("PrefDataPool: Size = " + this.totalSize);
    }

    //-----------------------------------------------------------------------
    // Get the message with the lowest seqNum for the given Object ID
    //-----------------------------------------------------------------------
    public OriginMsg peekLowestSeqNum(String objectID)
    {
	long seqNum = 0;
	OriginMsg dataMsg = null;
	LinkedList objectList = null;

	objectList = (LinkedList)this.objectIndexTable.get(objectID);
	if((objectList != null) && (!objectList.isEmpty())){
	    dataMsg = (OriginMsg)objectList.getFirst();
	    seqNum = dataMsg.getSeqNum();
	    assert(this.seqNumIndexTable.get(new Long(seqNum)) != null);
	}else{
	    dataMsg = null;
	}
	return(dataMsg);
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected long totalSize;
    protected Hashtable objectIndexTable;
    protected Hashtable seqNumIndexTable;
}
