//---------------------------------------------------------------------------
// PrefetchClient: Get prefetched data from the prefetch channel
//---------------------------------------------------------------------------
// For testing
import java.net.InetAddress;
import java.io.IOException;

public class PrefetchClient{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PrefetchClient(String originServerName, int portNum)
    {
	this.recvQueue = new SyncQueue();
	new PrefetchClientThread(originServerName, portNum, this.recvQueue);
    }

    //-----------------------------------------------------------------------
    // Dequeue the next message; block until one present
    //-----------------------------------------------------------------------
    public OriginMsg dequeue()
    {
	OriginMsg msg = null;

	msg = (OriginMsg)this.recvQueue.peekBlocked();
	this.recvQueue.dequeue();
	return(msg);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	PrefetchServer server = null;
	PrefetchClient client = null;
	PriorityQueue pq = null;
	PrioritySendThread sendThread = null;
	SyncLMCommPoint lmCommPoint = null;
	OriginMsg msg = null;

	try{
	    server = new PrefetchServer(4500);
	    pq = new PriorityQueue();
	    lmCommPoint = new SyncLMCommPoint(new LMCommPoint());
	    sendThread = new PrioritySendThread(pq, lmCommPoint);
	    server.addBinding(InetAddress.getByName("el-greco"), sendThread);

	    client = new PrefetchClient("el-greco", 4500);
	    pq.enqueue("testfile", 10, 0.5);
	    Thread.sleep(2000);
	    while(true){
		msg = client.dequeue();
		System.out.println("Type = " + msg.getMessageType());
		System.out.println("ObjectID = " + msg.getObjectID());
	    }
	}catch(IOException e){
	    System.err.println("" + e);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected SyncQueue recvQueue;
}
