//---------------------------------------------------------------------------
// PrefetchServer: Listens to connections asking for data to be prefetched
//---------------------------------------------------------------------------
import java.net.Socket;
import java.net.InetAddress;
import java.util.Hashtable;
import java.io.IOException;

public class PrefetchServer implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PrefetchServer(int portNum)
    {
	this.portNum = portNum;
	this.bindings = new Hashtable();
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	Socket niceSocket = null;
	InetAddress address = null;
	NiceServerSocket niceServerSocket = null;
	PrioritySendThread prioritySendThread = null;

	try{
	    niceServerSocket = new NiceServerSocket(this.portNum);
	    while(true){
		try{
		    prioritySendThread = null;
		    niceSocket = niceServerSocket.accept();
		    address = niceSocket.getInetAddress();
		    prioritySendThread = this.getBoundThread(address);
		    assert(prioritySendThread != null);
		    prioritySendThread.setSocket(niceSocket);
		}catch(IOException e){
		    System.err.println("" + e);
		    prioritySendThread.setSocket(null);
		}
	    }
	}catch(IOException e){
	    System.err.println("" + e);
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Add a new binding between the address and send thread
    //-----------------------------------------------------------------------
    public void addBinding(InetAddress address, PrioritySendThread sendThread)
    {
	this.bindings.put(address, sendThread);
	System.out.println("Adding binding: " + address);
    }

    //-----------------------------------------------------------------------
    // Return the thread bound to the given address
    //-----------------------------------------------------------------------
    protected PrioritySendThread getBoundThread(InetAddress address)
    {
	PrioritySendThread prioritySendThread = null;

	prioritySendThread = (PrioritySendThread)this.bindings.get(address);
	return(prioritySendThread);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	PrefetchServer ps = null;
	PriorityQueue pq = null;
	PrioritySendThread sendThread = null;

	try{
	    pq = new PriorityQueue();
	    ps = new PrefetchServer(4055);
	    sendThread = new PrioritySendThread(pq, new SyncLMCommPoint(new
		LMCommPoint()));
	    ps.addBinding(InetAddress.getByName("el-greco"),
			  sendThread);
	    pq.enqueue("testfile", 10, 0.5);
	}catch(IOException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int portNum;
    protected Hashtable bindings;
}
