//---------------------------------------------------------------------------
// PriorityQueue: Priority queue of objects
//---------------------------------------------------------------------------
import java.util.Hashtable;
import java.util.TreeSet;

public class PriorityQueue{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PriorityQueue()
    {
	this.indexTable = new Hashtable();
	this.treeSet = new TreeSet();
	this.bound = 0;
    }

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PriorityQueue(int bound)
    {
	this.indexTable = new Hashtable();
	this.treeSet = new TreeSet();
	this.bound = bound;
    }

    //-----------------------------------------------------------------------
    // Add an object to the priority queue
    //-----------------------------------------------------------------------
    public synchronized void enqueue(String objectID,
				     long seqNum,
				     double priority)
    {
	PQEntry pqe = null;
	boolean success = false;

	pqe = (PQEntry)this.indexTable.get(objectID);
	if(pqe != null){
	    success = this.treeSet.remove(pqe);
	    assert(success);
	}
	pqe = new PQEntry(objectID, seqNum, priority);
	success = this.treeSet.add(pqe);
	assert(success);
	this.indexTable.put(objectID, pqe);
	assert(this.indexTable.size() == this.treeSet.size());
	/*
	  System.out.println("PriorityQueue: Grew to " +
	  this.treeSet.size() +
	  " elements");
	*/
	this.notifyAll();
    }

    //-----------------------------------------------------------------------
    // Get the next object in the priority queue; block until one available
    //-----------------------------------------------------------------------
    public synchronized PQEntry peekNext()
    {
	while(this.treeSet.isEmpty()){
	    assert(this.indexTable.isEmpty());
	    try{
		this.wait();
	    }catch(InterruptedException e){
		assert(false);
	    }
	}
	assert(this.indexTable.size() == this.treeSet.size());
	return((PQEntry)this.treeSet.last());
    }

    //-----------------------------------------------------------------------
    // Dequeue the entry based on the object ID
    //-----------------------------------------------------------------------
    public synchronized PQEntry dequeue(String objectID)
    {
	PQEntry pqe = null;
	boolean success = false;

	pqe = (PQEntry)this.indexTable.get(objectID);
	if(pqe != null){
	    this.indexTable.remove(objectID);
	    success = this.treeSet.remove(pqe);
	    assert(success);
	}
	assert(this.indexTable.size() == this.treeSet.size());
	return(pqe);
    }

    //-----------------------------------------------------------------------
    // Program starting point
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	/*
	  PriorityQueue();
	  void enqueue(String objectID, long seqNum, double priority);
	  PQEntry peekNext();
	  PQEntry dequeue(String objectID);
	 */
	PriorityQueue pq = null;
	PQEntry pqe = null;
	int num = 0;

	pq = new PriorityQueue();
	for(int i = 1; i <= 50; i++){
	    pq.enqueue("" + i, 80, 1.0);
	    pq.enqueue("" + i, 10, 1.0);
	}

	while(!pq.indexTable.isEmpty()){
	    pqe = pq.peekNext();
	    pqe = pq.dequeue(pqe.getObjectID());
	    System.out.println("" + pqe.getObjectID());
	}
	assert(pq.treeSet.isEmpty());
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected Hashtable indexTable;
    protected TreeSet treeSet;
    protected int bound;
}
