//---------------------------------------------------------------------------
// PrioritySimThread: Simulate changing object priorities
//---------------------------------------------------------------------------
import java.io.IOException;
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.Hashtable;

public class PrioritySimThread implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PrioritySimThread(Hashtable priorityTable, String fileName)
	throws IOException
    {
	this.priorityTable = priorityTable;
	this.reader = new BufferedReader(new FileReader(fileName));
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	long delay = 0L;
	String line = null;
	int numEntries = 0;
	String objectID = null;
	double priority = 0.0;

	try{
	    do{
		delay = this.readDelay();
		if(delay >= 0){
		    try{
			Thread.sleep(delay);
		    }catch(InterruptedException e){
			System.err.println("" + e);
		    }
		    numEntries = this.readNumEntries();
		    for(int i = 0; i < numEntries; i++){
			line = this.reader.readLine();
			assert(line != null);
			objectID = this.getObjectID(line);
			priority = this.getPriority(line);
			this.priorityTable.put(objectID, new Double(priority));
		    }
		}
	    }while((delay >= 0) && (numEntries >= 0));
	    this.reader.close();
	}catch(IOException e){
	    assert(false);
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Read the delay from the file
    //-----------------------------------------------------------------------
    private long readDelay() throws IOException
    {
	long delay = 0;
	String line = null;

	delay = -1;
	line = this.reader.readLine();
	if(line != null){
	    delay = Long.parseLong(line);
	}
	return(delay);
    }

    //-----------------------------------------------------------------------
    // Read the number of entries before the next delay
    //-----------------------------------------------------------------------
    private int readNumEntries() throws IOException
    {
	int numEntries = -1;
	String line = null;

	line = this.reader.readLine();
	if(line != null){
	    numEntries = Integer.parseInt(line);
	}
	return(numEntries);
    }

    //-----------------------------------------------------------------------
    // Extract the object ID from the line
    //-----------------------------------------------------------------------
    private String getObjectID(String line)
    {
	int index = 0;

	index = line.indexOf(' ');
	return(line.substring(0, index));
    }

    //-----------------------------------------------------------------------
    // Extract the priority from the line
    //-----------------------------------------------------------------------
    private double getPriority(String line)
    {
	int index = 0;

	index = line.indexOf(' ');
	return(Double.parseDouble(line.substring(index + 1)));
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	try{
	    if(argv.length == 1){
		new PrioritySimThread(new Hashtable(), argv[0]);
	    }else{
		System.err.println("Usage: PrioritySimThread <filename>");
	    }
	}catch(IOException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected Hashtable priorityTable;
    protected BufferedReader reader;
}
