//---------------------------------------------------------------------------
// PriorityTracker: Keep track of object priorities
//---------------------------------------------------------------------------
import java.io.IOException;
import java.util.Hashtable;

public class PriorityTracker{

    //-----------------------------------------------------------------------
    // Constant
    //-----------------------------------------------------------------------
    public static final double DEFAULT_PRIORITY = 0.0;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public PriorityTracker(String fileName) throws IOException
    {
	this.priorityTable = new Hashtable();
	new PrioritySimThread(this.priorityTable, fileName);
    }

    //-----------------------------------------------------------------------
    // Return the priority of a given object
    //-----------------------------------------------------------------------
    public double getPriority(String objectID)
    {
	Double p = null;
	double priority = 0;

	p = (Double)this.priorityTable.get(objectID);
	if(p == null){
	    priority = DEFAULT_PRIORITY;
	}else{
	    priority = p.doubleValue();
	}
	return(priority);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	String objectID = null;
	PriorityTracker pt = null;

	try{
	    if(argv.length == 1){
		pt = new PriorityTracker(argv[0]);
		objectID = "Something.else";
		System.out.println("Priority of " + objectID + " = " +
				   pt.getPriority(objectID));
		objectID = "/u/nayate/research/Something";
		System.out.println("Priority of " + objectID + " = " +
				   pt.getPriority(objectID));
		Thread.sleep(1000);
		objectID = "Something.else";
		System.out.println("Priority of " + objectID + " = " +
				   pt.getPriority(objectID));
		objectID = "/u/nayate/research/Something";
		System.out.println("Priority of " + objectID + " = " +
				   pt.getPriority(objectID));
		Thread.sleep(1000);
		objectID = "Something.else";
		System.out.println("Priority of " + objectID + " = " +
				   pt.getPriority(objectID));
		objectID = "/u/nayate/research/Something";
		System.out.println("Priority of " + objectID + " = " +
				   pt.getPriority(objectID));
	    }else{
		System.err.println("Usage: PriorityTracker <filename>");
	    }
	}catch(IOException e){
	    System.err.println("PriorityTracker (main 1): " + e);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected Hashtable priorityTable;
}
