//---------------------------------------------------------------------------
// ReplicaListener: Listen to one replica for requests; one per replica
//---------------------------------------------------------------------------
import java.io.IOException;

public class ReplicaListener implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public ReplicaListener(ReplicaManager replicaManager,
			   String replicaName,
			   int regPort,
			   int ackPort,
			   SyncLMCommPoint lmComm) throws IOException
    {
	this.replicaManager = replicaManager;
	this.demReqQueue = new APMQReceiver(replicaName, regPort, ackPort);
	this.lmComm = lmComm;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	int lockToken = 0;
	String objectID = null;
	OriginMsg replicaMsg = null;
	AppMessage lockMsg = null;

	try{
	    while(true){
		replicaMsg = this.demReqQueue.dequeue();
		assert(replicaMsg.getMessageType() == AppMessage.READ);
		objectID = replicaMsg.getObjectID();
		lockMsg = this.lmComm.requestSharedLock(objectID);
		assert(lockMsg.getMessageType() ==
		       AppMessage.LMREP_GRANTED_SHARED);
		lockToken = lockMsg.getLockToken();
		this.replicaManager.enqueueDemandReply(objectID, lockToken);
		this.demReqQueue.sendAck(replicaMsg.getSeqNum());
		this.lmComm.releaseLock(objectID, lockToken);
	    }
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected ReplicaManager replicaManager;
    protected APMQReceiver demReqQueue;
    protected SyncLMCommPoint lmComm;
}
