//---------------------------------------------------------------------------
// Statistics: Used to gather and print statistics
//---------------------------------------------------------------------------
public class Statistics implements Runnable{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    protected static final long STATS_PRINT_INTERVAL = 30000;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public Statistics()
    {
	this.bwReg = 0;
	this.bwNice = 0;
	this.bwRegData = 0;
	this.bwNiceData = 0;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	while(true){
	    try{
		Thread.sleep(Statistics.STATS_PRINT_INTERVAL);
		// Time to print statistics information
		Globals.statPrintln("REG bw consumed = " + this.bwReg);
		Globals.statPrintln("NICE bw consumed = " + this.bwNice);
		Globals.statPrintln("REG data bw consumed = " +
				    this.bwRegData);
		Globals.statPrintln("NICE data bw consumed = " +
				    this.bwNiceData);
		Globals.statPrintln("");
	    }catch(InterruptedException e){
		System.err.println("" + e);
	    }
	}
    }

    //-----------------------------------------------------------------------
    // Increment the bandwidth consumed by the regular TCP socket
    // NOTE: Re-sent packets will get counted multiple times
    //-----------------------------------------------------------------------
    public synchronized void incRegBW(long bw)
    {
	this.bwReg += bw;
    }

    //-----------------------------------------------------------------------
    // Increment the bandwidth consumed by the TCP-Nice socket
    // NOTE: Re-sent packets will get counted multiple times
    //-----------------------------------------------------------------------
    public synchronized void incNiceBW(long bw)
    {
	this.bwNice += bw;
    }

    //-----------------------------------------------------------------------
    // Increment the bandwidth consumed by the regular TCP socket
    // NOTE: Re-sent packets will get counted multiple times
    //-----------------------------------------------------------------------
    public synchronized void incRegDataBW(OriginMsg msg)
    {
	byte[] data = null;

	if(msg.getMessageType() == AppMessage.DATA){
	    data = msg.getData();
	    this.bwRegData += data.length;
	}
    }

    //-----------------------------------------------------------------------
    // Increment the bandwidth consumed by the TCP-Nice socket
    // NOTE: Re-sent packets will get counted multiple times
    //-----------------------------------------------------------------------
    public synchronized void incNiceDataBW(OriginMsg msg)
    {
	byte[] data = null;

	if(msg.getMessageType() == AppMessage.DATA){
	    data = msg.getData();
	    this.bwNiceData += data.length;
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected long bwReg;
    protected long bwNice;
    protected long bwRegData;
    protected long bwNiceData;
}
