//---------------------------------------------------------------------------
// SyncLMCommPoint: Synchronous interface to the lock manager
//---------------------------------------------------------------------------
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

public class SyncLMCommPoint{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public SyncLMCommPoint(LMCommPoint lmCommPoint)
    {
	this.lmCommPoint = lmCommPoint;
    }

    //-----------------------------------------------------------------------
    // Request a shared lock on the passed-in object
    //-----------------------------------------------------------------------
    public AppMessage requestSharedLock(String objectID)
    {
	LMMailBox mailBox = null;

	mailBox = new LMMailBox();
	this.lmCommPoint.sendSharedLockRequest(objectID, mailBox);
	return(mailBox.waitLockMsg());
    }

    //-----------------------------------------------------------------------
    // Request an exclusive lock on the passed-in object
    //-----------------------------------------------------------------------
    public AppMessage requestExclusiveLock(String objectID)
    {
	LMMailBox mailBox = null;

	mailBox = new LMMailBox();
	this.lmCommPoint.sendExclusiveLockRequest(objectID, mailBox);
	return(mailBox.waitLockMsg());
    }

    //-----------------------------------------------------------------------
    // Release a lock
    //-----------------------------------------------------------------------
    public void releaseLock(String objectID, int lockToken)
    {
	this.lmCommPoint.releaseLock(objectID, lockToken);
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	// SyncLMCommPoint.test1();
	SyncLMCommPoint.test2();
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void test1()
    {
	AppMessage msg1 = null;
	AppMessage msg2 = null;
	SyncLMCommPoint lmCommPoint = null;

	lmCommPoint = new SyncLMCommPoint(new LMCommPoint());
	msg1 = lmCommPoint.requestSharedLock("someObj");
	System.out.println("Received token: " + msg1.getLockToken());
	msg2 = lmCommPoint.requestSharedLock("someObj");
	System.out.println("Received token: " + msg2.getLockToken());
	//lmCommPoint.releaseLock("someObj", msg1.getLockToken());
	//System.out.println("Freed token: " + msg1.getLockToken());
	msg1 = lmCommPoint.requestSharedLock("someObj");
	System.out.println("Received token: " + msg1.getLockToken());
    }

    //-----------------------------------------------------------------------
    // Used for testing
    //-----------------------------------------------------------------------
    public static void test2()
    {
	new SyncLMTestThread1();
	new SyncLMTestThread2();
	new SyncLMTestThread3();
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected LMCommPoint lmCommPoint;
}

//---------------------------------------------------------------------------
// Thread 1 (used for testing)
//---------------------------------------------------------------------------
class SyncLMTestThread1 implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public SyncLMTestThread1()
    {
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	// Thread 1
	int lock = 0;
	AppMessage msg = null;
	SyncLMCommPoint point = null;

	point = new SyncLMCommPoint(new LMCommPoint());
	System.out.println("Thread 1 asking for the shared lock");
	msg = point.requestSharedLock("");
	lock = msg.getLockToken();
	System.out.println("Thread 1 got the shared lock: " + lock);
	try{
	    Thread.sleep(3000);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
	System.out.println("Thread 1 releasing the shared lock: " + lock);
	point.releaseLock("", lock);
    }
}

//---------------------------------------------------------------------------
// Thread 2 (used for testing)
//---------------------------------------------------------------------------
class SyncLMTestThread2 implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public SyncLMTestThread2()
    {
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	// Thread 2
	int lock = 0;
	AppMessage msg = null;
	SyncLMCommPoint point = null;

	point = new SyncLMCommPoint(new LMCommPoint());
	try{
	    Thread.sleep(1000);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
	System.out.println("Thread 2 asking for the exclusive lock");
	msg = point.requestExclusiveLock("");
	lock = msg.getLockToken();
	System.out.println("Thread 2 got the exclusive lock: " + lock);
	System.out.println("Thread 2 releasing the exclusive lock: " + lock);
	point.releaseLock("", lock);
    }
}

//---------------------------------------------------------------------------
// Thread 3 (used for testing)
//---------------------------------------------------------------------------
class SyncLMTestThread3 implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public SyncLMTestThread3()
    {
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	// Thread 3
	int lock = 0;
	AppMessage msg = null;
	SyncLMCommPoint point = null;

	point = new SyncLMCommPoint(new LMCommPoint());
	try{
	    Thread.sleep(2000);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
	System.out.println("Thread 3 asking for the shared lock");
	msg = point.requestSharedLock("");
	lock = msg.getLockToken();
	System.out.println("Thread 3 got the shared lock: " + lock);
	try{
	    Thread.sleep(2000);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}
	System.out.println("Thread 3 releasing the shared lock: " + lock);
	point.releaseLock("", lock);
    }
}
