//---------------------------------------------------------------------------
// TCPThread: Implementation of the thread listening to a TCP channel
//---------------------------------------------------------------------------
public class TCPThread extends AtomicThread{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public TCPThread(OriginQueueFSM fsm,
		     APMQReceiver demInvChannel,
		     BooleanLock lock)
    {
	super(lock);
	this.fsm = fsm;
	this.channel = demInvChannel;
	this.startSelf();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	int msgType = 0;
	long prevSeqNum = -1;
	OriginMsg msg = null;

	try{
	    while(true){
		msg = this.channel.dequeue();
		msgType = msg.getMessageType();
		assert((msgType == AppMessage.INV) ||
		       (msgType == AppMessage.DATA));
		assert((msg.getSeqNum() == (prevSeqNum + 1)) ||
		       (prevSeqNum < 0));
		prevSeqNum = msg.getSeqNum();
		if(msgType == AppMessage.INV){
		    this.enterCriticalSection();
		    this.fsm.handleInvalidate(msg);
		    this.exitCriticalSection();
		}else{
		    this.enterCriticalSection();
		    this.fsm.handleDemandData(msg);
		    this.exitCriticalSection();
		}
	    }
	}catch(Throwable t){
	    t.printStackTrace();
	    assert(false);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected OriginQueueFSM fsm;
    protected APMQReceiver channel;
}
