//---------------------------------------------------------------------------
// OMQTest: Test the OriginMsgQueue class
//---------------------------------------------------------------------------
import java.io.IOException;

public class OMQTest{

    //-----------------------------------------------------------------------
    // Used to test the origin message queue
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	OriginMsg originMsg = null;
	OriginQueueFSM fsm = null;
	SyncQueue localQueue = null;
	APMQSender demReqChannel = null;
	APMQReceiver demInvChannel = null;
	MQReceiver prefChannel = null;
	BooleanLock syncLock = null;

	try{
	    syncLock = new BooleanLock();
	    localQueue = new SyncQueue();
	    demReqChannel = new APMQSender("localhost", 1809, 1810);
	    demInvChannel = new APMQReceiver("localhost", 1909, 1910);
	    prefChannel = new MQReceiver("localhost", 2009);
	    fsm = new OriginQueueFSM(syncLock,
				     localQueue,
				     5000,
				     demReqChannel,
				     demInvChannel,
				     prefChannel);
	    new FakeOriginServer(fsm, syncLock);
	    while(true){
		originMsg = (OriginMsg)localQueue.peekBlocked();
		localQueue.dequeue();
		System.out.println("Message arrived:");
		System.out.println("seqNum = " + originMsg.getSeqNum());
		System.out.println("type = " + originMsg.getMessageType());
		System.out.println("objID = " + originMsg.getObjectID());
		System.out.println("Current time = " +
				   System.currentTimeMillis());
		System.out.println();
		Assert.affirm(!originMsg.containsLock());
	    }
	}catch(IOException e){
	    System.err.println("" + e);
	}
    }
}

//---------------------------------------------------------------------------
// Fake the origin server
//---------------------------------------------------------------------------
class FakeOriginServer implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FakeOriginServer(OriginQueueFSM fsm, BooleanLock syncLock)
    {
	try{
	    this.demReqChannel = new APMQReceiver("localhost", 1809, 1810);
	    this.demInvChannel = new APMQSender("localhost", 1909, 1910);
	    this.prefChannel = new MQSender("localhost", 2009);
	    this.fsm = fsm;
	    this.syncLock = syncLock;
	    (new Thread(this)).start();
	}catch(IOException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	// Send an invalidate to see if we eventually receive it.
	long seqNum = 0;
	OriginMsg msg = null;
	try{
	    System.out.println("Current time = " + System.currentTimeMillis());
	    seqNum = this.demInvChannel.getSeqNum() + 1;

	    // Send some data to increase the seqnum
	    this.demInvChannel.enqueue(new OriginMsg(seqNum,
						     AppMessage.DATA,
						     "seqRaiser",
						     new byte[1]));

	    // Send a request to switch to roll mode
	    AtomicThread.acquireCSLock(this.syncLock);
	    this.fsm.startRollMode("o", 10);
	    AtomicThread.releaseCSLock(this.syncLock);

	    /*
	    // Make sure we get a demand request
	    msg = this.demReqChannel.dequeue();
	    System.out.println("Message: ");
	    System.out.println("Type = " + msg.getMessageType());
	    System.out.println("seqNum = " + msg.getSeqNum());
	    System.out.println("objectID = " + msg.getObjectID());
	    System.out.println();
	    */

	    /*
	    // Send invalidate with higher seqNum
	    this.demInvChannel.enqueue(new OriginMsg(seqNum + 1,
						     AppMessage.INV,
						     "o"));

	    // Send data with higher seqnum than asked, but lower than known
	    this.prefChannel.enqueue(new OriginMsg(11,
						   AppMessage.DATA,
						   "o",
						   new byte[1]));

	    System.out.println("Sleeping");
	    Thread.sleep(3000);

	    // Send data with correct seqnum to accept
	    this.prefChannel.enqueue(new OriginMsg(seqNum + 1,
						   AppMessage.DATA,
						   "o",
						   new byte[1]));
	    */

	    /*
	    this.prefChannel.enqueue(new OriginMsg(seqNum + 10,
						   AppMessage.DATA,
						   "a",
						   new byte[1]));
	    this.prefChannel.enqueue(new OriginMsg(seqNum,
						   AppMessage.DATA,
						   "a",
						   new byte[1]));
	    */
	    /*
	    System.out.println("Now sending old PREF");
	    this.demInvChannel.enqueue(new OriginMsg(seqNum + 1,
						     AppMessage.INV,
						     "b",
						     new byte[1]));
	    */
	}catch(IOException e){
	    System.err.println("" + e);
	}catch(ReplicaUnreceptiveException e){
	    System.err.println("" + e);
	}/*catch(InterruptedException e){
	    System.err.println("" + e);
	    }*/finally{
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected APMQReceiver demReqChannel;
    protected APMQSender demInvChannel;
    protected MQSender prefChannel;
    protected OriginQueueFSM fsm;
    protected BooleanLock syncLock;
}
