//---------------------------------------------------------------------------
// ReplicaTest: Test the Replica class
//---------------------------------------------------------------------------
import java.io.IOException;
import java.net.Socket;

public class ReplicaTest{

    //-----------------------------------------------------------------------
    // Program starting point
    //-----------------------------------------------------------------------
    public static void main(String[] argv)
    {
	ReplicaFSM replicaFSM = null;

	if(argv.length == 2){
	    if(argv[0].equals("server")){
		new FakeOriginServer(argv[1],
				     1809,
				     1776,
				     1978,
				     15000L);
	    }else if(argv[0].equals("client")){
		try{
		    replicaFSM = new ReplicaFSM(argv[1],
						1809,
						1776,
						1978,
						15000);
		    new FSCommServer(replicaFSM);
		}catch(IOException e){
		    System.err.println("" + e);
		    e.printStackTrace();
		}
	    }
	}else{
	    System.err.println("Usage: ReplicaTest <client|server> <name>");
	}
    }
}

//---------------------------------------------------------------------------
// Fake server used to test the replica
//---------------------------------------------------------------------------
class FakeOriginServer implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FakeOriginServer(String serverName,
			    int tcpPort,
			    int ackPort,
			    int nicePort,
			    long timeout)
    {
	this.serverName = serverName;
	this.tcpPort = tcpPort;
	this.ackPort = ackPort;
	this.nicePort = nicePort;
	this.timeout = timeout;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	long seqNum = 0;
	APMQSender tcpChannel = null;
	APMQReceiver demReqChannel = null;
	MQSender prefChannel = null;

	try{
	    tcpChannel = new APMQSender(serverName, tcpPort, ackPort);
	    demReqChannel = new APMQReceiver(serverName, tcpPort, ackPort);
	    prefChannel = new MQSender(serverName, nicePort);

	    seqNum = tcpChannel.getSeqNum() + 1;

	    // Send the message to get seqnum updated
            tcpChannel.enqueue(new OriginMsg(seqNum,
					     AppMessage.DATA,
					     "seqRaiser",
					     new byte[1]));

	    // Make sure seqRaiser is written
	    Thread.sleep(1000);

	    // Start the fake demand request handler
	    new FakeRequestHandler(demReqChannel, tcpChannel, seqNum + 1);

	    // Send a demand read
	    new FakeFSRequest(serverName, "a", 2);

	    // Wait for the demand read to reach
	    Thread.sleep(1000);

	    // Send a demand object, but not the requested one.
	    tcpChannel.enqueue(new OriginMsg(seqNum + 1,
					     AppMessage.DATA,
					     "b",
					     new byte[1]));

	    /*
	    // Send data
	    prefChannel.enqueue(new OriginMsg(seqNum,
					      AppMessage.DATA,
					      "a",
					      new byte[1]));

	    // Send invalidate
            tcpChannel.enqueue(new OriginMsg(seqNum + 2,
					     AppMessage.INV,
					     "a",
					     new byte[1]));
	    */

	}catch(IOException e){
	    System.err.println("" + e);
	}catch(ReplicaUnreceptiveException e){
	    System.err.println("" + e);
	}catch(InterruptedException e){
	    System.err.println("" + e);
	}finally{
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected String serverName;
    protected int tcpPort;
    protected int ackPort;
    protected int nicePort;
    protected long timeout;
}

//---------------------------------------------------------------------------
// Emulate the origin server request handler
//---------------------------------------------------------------------------
class FakeRequestHandler implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FakeRequestHandler(APMQReceiver demChannel,
			      APMQSender repChannel,
			      long seqNum)
    {
	this.demChannel = demChannel;
	this.repChannel = repChannel;
	this.seqNum = seqNum;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Main thread method
    //-----------------------------------------------------------------------
    public void run()
    {
	OriginMsg msg = null;

	try{
	    while(true){
		msg = (OriginMsg)this.demChannel.dequeue();
		System.out.println("Got a replica message:");
		System.out.println("type = " + msg.getMessageType());
		System.out.println("oid = " + msg.getObjectID());
		/*
		this.repChannel.enqueue(new OriginMsg(this.seqNum,
						      AppMessage.DATA,
						      msg.getObjectID(),
						      new byte[1]));
		*/
	    }
	}/*catch(ReplicaUnreceptiveException e){
	    System.out.println("" + e);
	}catch(IOException e){
	    System.out.println("" + e);
	    }*/finally{
	    }
    }

    //-----------------------------------------------------------------------
    // ProtectedData
    //-----------------------------------------------------------------------
    protected APMQReceiver demChannel;
    protected APMQSender repChannel;
    protected long seqNum;
}

//---------------------------------------------------------------------------
// Emulate the file system read request
//---------------------------------------------------------------------------
class FakeFSRequest implements Runnable{

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public FakeFSRequest(String serverName, String objectID, int lockToken)
    {
	this.serverName = serverName;
	this.objectID = objectID;
	this.lockToken = lockToken;
	(new Thread(this)).start();
    }

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public void run()
    {
	Socket s = null;
	FSCommPacket p = null;

	try{
	    s = new Socket(serverName, FSCommServer.PORT_NUM);
	    p = new FSCommPacket(AppMessage.READ, lockToken, objectID, s);
	    p.sendSelf(s.getOutputStream());

	    p = new FSCommPacket(s.getInputStream(), s);
	    Assert.affirm(p.getMessageType() == AppMessage.ACK);
	    s.close();
	}catch(IOException e){
	    System.err.println("" + e);
	}
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected String serverName;
    protected String objectID;
    protected int lockToken;
}
