//---------------------------------------------------------------------------
// LMMailBox: Used to wait for locks
//---------------------------------------------------------------------------
public class LMMailBox implements LMCallBack{

    //-----------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------
    public static final int LM_SHARED_LOCK = 1;
    public static final int LM_EXCLUSIVE_LOCK = 2;

    //-----------------------------------------------------------------------
    // Constructor
    //-----------------------------------------------------------------------
    public LMMailBox(String objectID, int lockType)
    {
	this.packet = null;
	this.objectID = objectID;
	this.lockType = lockType;
	assert((this.lockType == LM_SHARED_LOCK) ||
	       (this.lockType == LM_EXCLUSIVE_LOCK));
    }

    //-----------------------------------------------------------------------
    // Wait for a lock token to arrive
    //-----------------------------------------------------------------------
    public synchronized LMReplyPacket waitLockToken()
    {
	while(this.packet != null){
	    try{
		this.wait();
	    }catch(InterruptedException e){
		System.err.println("" + e);
	    }
	}
	return(this.packet);
    }

    //-----------------------------------------------------------------------
    // Called when 
    //-----------------------------------------------------------------------
    public synchronized void lockArrived(LMReplyPacket packet)
    {
	assert(this.objectID.equals(packet.getObjectID()));
	if(this.lockType == LM_SHARED_LOCK){
	    assert(packet.getMessageType() == AppMessage.LMREP_GRANTED_SHARED);
	}else{
	    assert(packet.getMessageType() ==
		   AppMessage.LMREP_GRANTED_EXCLUSIVE);
	}
	this.packet = packet;
	this.notifyAll();
    }

    //-----------------------------------------------------------------------
    // Protected data
    //-----------------------------------------------------------------------
    protected int lockType;
    protected String objectID;
    protected LMReplyPacket packet;
}
